@@ -1121,14 +1121,15 @@ class DBDataset(DBObject):
             raise
         return self
 
-    def list_files(self, with_metadata=False, limit=None):
+    def list_files(self, with_metadata=False, limit=None, include_retired_files=False):
         meta = "null as metadata" if not with_metadata else "f.metadata"
         limit = f"limit {limit}" if limit else ""
+        retired = "" if include_retired_files else "and not f.retired"
         sql = f"""select f.id, f.namespace, f.name, {meta}, f.size, f.checksums, f.creator, f.created_timestamp 
                     from files f
                         inner join files_datasets fd on fd.file_id = f.id
                     where fd.dataset_namespace = %s and fd.dataset_name=%s
-                        and not f.retired
+                        {retired}
                     {limit}
         """
         c = self.DB.cursor()
