@@ -394,7 +394,7 @@ class MetaCatClient(HTTPClient, TokenAuthClientMixin):
         except NotFoundError:
             return None
 
-    def get_dataset_files(self, did, namespace=None, name=None, with_metadata=False):
+    def get_dataset_files(self, did, namespace=None, name=None, with_metadata=False, include_retired_files=False):
         """Gets single dataset
         
         Parameters
@@ -413,7 +413,9 @@ class MetaCatClient(HTTPClient, TokenAuthClientMixin):
             did = namespace + ':' + name
         try:
             with_metadata = "yes" if with_metadata else "no"
-            return self.get_json_stream(f"data/dataset_files?dataset={did}&with_metadata={with_metadata}")
+            include_retired_files = "yes" if include_retired_files else "no"
+            url = f"data/dataset_files?dataset={did}&with_metadata={with_metadata}&include_retired_files={include_retired_files}"
+            return self.get_json_stream(url)
         except NotFoundError:
             return None
         
